#!/usr/bin/python3
# -*- coding: utf-8 -*-

from funnylog2 import log as log
from funnylog2 import logger as logger

from youqu3._setting import setting as setting
from youqu3.sleepx import sleep
from youqu3.__version__ import __version__ as version

__all__ = ['setting', 'log', 'logger', 'sleep', 'version']
